%macro build(location=, table=, outtype=view, outlib=work, outname=, obs=, labels=y);

/* make sure these variables exist, the values must be set outside the macro */
%global hrvurl wh user pw urlopts debug source mprint notes userobslimit;

%if %quote(&location) = ? or %upcase(%quote(&location))=HELP or %quote(&table)= or
    %quote(&hrvurl)=%str() or %quote(&wh)=%str() or %quote(&user)=%str() %then %do;
    %put build(location=public, table=, outtype=view, outlib=work, outname=);
    %put ;
    %put Parameters:;
    %put  ** LOCATION   - public or private.  If missing, this defaults to DWDATA;
    %put  ** TABLE      - the table to make the view/data set from. Required.;
    %put  ** OUTTYPE    - the type of output (table or view).;
    %put  ** OUTLIB     - the output library (work, prvdata, another user supplied libname).;
    %put  ** OUTNAME    - name of the output data set/ view.;
    %put ;
    %put  ***** the following have to be set prior to calling this macro *****.;
    %put  ** HRVURL     - the url that references where HR Vision is installed.(Should include HTTP: protocol);
    %put  ** WH         - the warehouse id that is being used.;
    %put  ** USER       - the user id of the person running this.;
    %put  ** PW         - the users password for HR Vision for the specified warehouse.;
%end;
%else %do;
    %local char lastchar secsubset keyvar index smsgstr msgstr ncols comma tmp i colparm dspwd col n tmp 
           initcode masterSecTable encodedPw dwloc pwstring;

    %if %quote(&outname) = %str() %then %do;
        /* default the outname to the table.*/
        %let outname=&table;
    %end;

    /* if the userobslimit is set and its less than the obs= parameter, then reset the obs= parameter */
    /* to userobslimit */
    %if %quote(&userobslimit) ne %str() %then %do;
       %if %quote(&obs) = %str() %then %let obs=&userobslimit;
       %else %if %quote(&userobslimit) lt %quote(&obs) %then %let obs=&userobslimit;
    %end;

    /* clean up the input parameters if necessary*/
        %let length=%length(&hrvurl);
        %let lastchar=%substr(&hrvurl, &length, 1);
    %if %superq(lastchar) ne %str(/) %then %do;
        %let hrvurl = %str(&hrvurl)/;
    %end;
        %if %quote(&outlib)=%str() %then %let outlib=work;
        %if %quote(&outtype)=%str() %then %let outtype=view;

    %let location = %upcase(&location);
    %if %quote(&location)=PUBLIC or %quote(&location)=PUBDATA %then %do;
        %let location = PUBDATA;
    %end;
        %else %if %quote(&location)=PRVDATA or %quote(&location)=PRIVATE %then %do;
                %let location=PRVDATA;
        %end;
        %else %let location=DWDATA;
        
        %if &debug=Y %then %put NOTE: LOCATION has been set to &location;

        /* call the scramble macro with the pw to get the scrabmled pw */
        %scramble(&pw);
        
        /*************************************************************
	 special fix to handle problems with accessing HTTPS sites
	 - Set a macro variable HTTPSFIX to Y to allow this fix to work
	 - Set a macro variable HTTPSFIX_URL to a desired HRV server to 
	       access metadata.
	 - If HTTPSFIX_URL is not set, then the original HTTPS url is 
	       used, except the HTTPS will be changed to HTTP.
	 ***************************************************************/
		          
	%global httpsfix httpsfix_url;
	%local httpsindex;
	%if %qupcase(&httpsfix) = Y %then %do;
	   %if %bquote(&httpsfix_url)=%str() %then %do;
		%let httpsindex = %index(&hrvurl, https);
		%if &httpsindex GE 2 %then
		%let hrvurl=%substr(&hrvurl,1,&httpsindex-1)http%substr(&hrvurl,&httpsindex+5);	
		%else %if &httpsindex = 1 %then
		   %let hrvurl=http%substr(&hrvurl,&httpsindex+5);
		%else %put WARNING: HTTPS Fix is turned on but supplied url does not contain HTTPS.;
	  %end;
	  %else %let hrvurl=%bquote(&httpsfix_url);
	%end;
	/****************************************************************
	    end of https fix
	 ****************************************************************/
		
        /* define the filename to the getTableData JSP in the HRV install area */
        %let url = &hrvurl.Jsp/getTableData.jsp?curWH=&wh.%nrstr(&user=)&user%nrstr(&pw=)&encryptPw%nrstr(&table=)&table%nrstr(&loc=)&location%nrstr(&labels=)&labels;

        /* blank out the macro variable for the encrypted pw. */
        %let encryptPw=;

        %let ncols=0;
    /* read the results of the filename */

        /* delete any table with the same name as the target table. This should 
                always happen, even if the target table is not sucessfully recreated. */
        %deltable(tables=&outlib..&table, password=&datapwd);
        
    filename web url "&url" &urlopts;
    data _null_;
        infile web pad lrecl=12000 length=l;
        retain alphas 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_';
        retain nums '01234567890';
        length char $1  keyvar $32 keyval $12000;
        retain delim ':=';
        input;
                %if &debug=Y %then %do;
                    length _thisline $12000;

                    _thisline=trim(_infile_);
        put 'line' _n_ 4. '=' _thisline;
                %end;

                /* delete any line not containing the delimiter */
                    delimpos = index(_infile_,delim);
                    if delimpos gt 0;
       
        /* do some simple checking to make sure we have a valid SAS variable name*/
        keyvar=substr(_infile_,1,delimpos-1);
        length=length(keyvar);
        if length ge 1 and length le 32;
        char=substr(keyvar,1,1);
        if index(alphas,char) gt 0;
        do i=2 to length(keyvar);
            char=substr(keyvar,i,1);
            if index(alphas||nums,char) gt 0;
        end;

        /* put out the macro variable and value */
        keyval=substr(_infile_,delimpos+length(delim));
        call symput(keyvar, trim(keyval));
        %if &debug=Y %then %str(put '         ' keyvar '=' keyval;);
    run;
    
    /* decrypt the dspwd. */
    %unscramble(%bquote(&dspwd));
    %let dspwd=&decryptPw;

    options notes &source;
    /* submit the libname for views if needed */
    &initcode;  

    /* if there are no columns then there is nothing to do */
    %if &ncols le 0 %then %do; 
      %put ERROR: Did not build table &outlib..&outname.;
	  %goto finish;
    %end;

    /* open the base data set.  If it doesn't open, then stop processing */ 
	%if &dspwd ne %str() %then %let pwstring=(read=&dspwd);
	%let dsid=%sysfunc(open(&location..&dsname&pwstring));
	%if &dsid le 0 %then %do;
	   %put ERROR: Cannot open base data set &location..&dsname to validate columns.;
	   %goto finish;
	%end; 

	/* for each column, check it's existence */
    %do i = 1 %to &ncols; 
       /* get the variable name as the first word of the column string */
	   %let varname=%scan(&&col&i,1,%str( )); 
       %if &varname ne %str( ) %then %do;
          %let varnum=%sysfunc(varnum(&dsid,&varname)); 
	      %if %sysfunc(varnum(&dsid,&varname)) le 0 %then %do;
		     %put Warning: column &varname does not exist in table &location..&dsname..;
             %let col&i=;
		  %end;
		  %else %do;
             %let tmp = &tmp&comma%str(&&col&i); 
		  	 %let comma=,;
    	  %end; 
	   %end; 			
     %end;  
 
	 %let dsid=%sysfunc(close(&dsid));
		 
     %let colparm = %superq(tmp);
 
     /* if everything is successful then generate the desired code */
     %if %quote(&debug) ne Y %then %str(options nosource;);
     %if &colparm ne %str() %then
          %generate(%qtrim(&location), %qtrim(&dsname), %qtrim(&dspwd), %qtrim(%bquote(&secsubset)), , %qtrim(&outname), %qtrim(&outlib), %qtrim(&outtype), %qtrim(&keyvar), %bquote(&smsgstr), %bquote(&msgstr), %nrquote(&colparm), 
                        baseds=&masterSecTable, obs=&obs);
	 %else %do;
        %put WARNING: No available columns in base dataset &location..&dsname..;
	    %goto finish;
     %end;
     options &source; 
%end;
%finish:
%mend;