/****************************************************************
* Id			- the type of file to export.
* localhost		- the localhost value for use when exporting to a socket.
* port			- the port value for use when exporting to a socket.
* xprtfile 	- the actual file that we want to create.
* currview	- the current view that we are exporting from.
* tabledesc	- the description of the table.
* tableName - the name of the access table to create.
* subset		- the value of the subset on the table being exported.
* labels		- on or off, default to on.
* varnames	- shown or hidden, default to shown.
******************************************************************/
%macro export(Id=, localhost=, port=, xprtfile=, currview=, tabledesc=, tblName=, subset=, labels=on, varnames=shown, xlsfmts=y);
   %global debug;
	%local Id localhost port xprtfile currview tabledesc tableName subset labels varnames; 
	%if &debug ne Y %then %str(options nomprint nosymbolgen nomlogic;);

	%let senttosock=N;
	/* set up the appropriate filename....*/
	%if &Id=DLM OR &Id=CSV OR &Id=TAB OR &Id=PDF %then %do;
		* these export types go to a socket.;
		filename sock SOCKET "&localhost:&port" lrecl=32767;
		%let senttosock=Y;
	%end;
	%else %do;
		* the other export types will be exported to a file.;
		filename sock "&xprtfile";
		* remove the file first so the various SAS procedures can write the file out.;
		%let sysrc=%sysfunc(fdelete(sock));
	%end;
	
	/* actually do the export for the various types */
	%if &Id=ACCESS OR &Id=ACCESS97 %then %do;
		proc export data=&currview DBMS=&Id outtable="&tblName" replace;
		database=sock;
		run;
	%end;
	%else %if &Id = PDF %then %do;
		ods listing close;
		ods pdf file=sock notoc;
		options nodate nonumber;
		title1 "&tabledesc ";

		%if &subset NE %then %do;
			%str(title2 "&subset";);
		%end;
		%RPTGEN(dataset=&currview,columns=_ALL_,group=, total=, rptopts=, rpttype=D, grandanalysis=N, subanalysis=N);
		ods pdf close;
		ods listing;
		run;
		goptions reset=all;
	%end;
	%else %if &Id=EXCEL5 OR &Id=EXCEL97 %then %do;
		%if %substr(%upcase(&xlsfmts),1,1)=Y %then %do;                                          
	      		%fmtout(&currview,&currview.2);
	      		%let currview=&currview.2;
		%end;
		
		%local n maxcount;
		%let maxcount=16384 ;
		%countrows(viewname=&currview,maxcount=&maxcount);
		%if &debug=Y %then %put &currview._count;
                data _null_;
		  set &currview._count;
		  %if &debug=Y %then %str(put n=;);
		  call symput('n',put(n,8.));
		run;
		
		/* added code on 8-26-2004 to force the PROC EXPORT to work when the macro variable */
		/*  AlwaysUseExport is set to 1 */
		%global AlwaysUseExport;
		%if %quote(&AlwaysUseExport)=1 %then %let maxcount=0;
		/* end of new code */
		%if &n ge &maxcount %then %do;
			proc export data=&currview DBMS=&Id outfile=sock replace;
			run;
		%end;
		%else %do;	
		        %global dbloadOptions;
			%if &Id=EXCEL5 %then %let xlsvrsn=5;
			%else %let xlsvrsn=7;
			proc dbload data=&currview dbms=XLS;
			        &dbloadOptions
				path="sock";                                                       
				version=&xlsvrsn;
				%if &varnames=shown %then %do;
					putnames=Y;
					%if &labels=on %then %do;
						label;
					%end;
				%end;
				reset ALL;
				limit=0;
				load;
			run;
            	%end;
	%end;
	%else %do;
		proc export data=&currview DBMS=&Id outfile=sock replace;
		run;
	%end;
	
	%if &senttosock=N %then %do;
		filename sock "&xprtfile" recfm=f;
		filename _sockout SOCKET "&localhost:&port" recfm=f;
		data _null_;infile sock;input;file _sockout;put _infile_;run;
		filename _sockout clear;
	%end;

	filename sock clear;
%mend;



