/****************************************************************
* location	- pubdata or prvdata.
* table		- table name you are trying to display.
* pwd			- table password needed.
* secsub 	- subset beans where clause (where statement) for the security subset.
* subclaus	- subset beans where clause (where statement).
* outname	- output data sets name (view3, etc.).
* outlib		- output libname (usually work).
* outtype	- table or view.
* keyvar		- the key variable from the table you want to display, used for merging.
* indexon	- the index var line that goes in a data step (hrvindex = _N_).
* secmsgst	- message to display when sec. subset can not be applied.
* dnemsgst	- message to display when the table to display does not exist.
* bigcolst	- the big string that goes in the final sql step that defines all the columns.
******************************************************************/
%macro visGen(location, table, pwd, secsub, subclaus, outname, outlib, outtype, 
keyvar, indexon, secmsgst, dnemsgst, bigcolst, baseds=empmast, obs=);  
	%let dbg=;
	/* %let dbg = options mprint; */
	&dbg;
	%if &baseds = %then %let baseds=empmast;
	%let dsname=&location..&table;
	%let pass=pw="&pwd";
	%let secwhere=;
	%let where=;
	%if %length(%trim(&secsub)) > 0 %then %do;
		%let secwhere=where=(&secsub); 
	%end;

	%if &obs ne %then 
	  %let obs=obs=&obs;
	
	%if %length(%trim(&subclaus)) > 0 %then %do;
		%let where=where &subclaus;
	%end;
	%if %sysfunc(exist(&dsname)) OR %sysfunc(exist(&dsname, view)) %then %do;
		* data set exists, so proceed.;
		* apply the security subset.;
		%let rc = 1;
		data &outName.Temp;
			set &dsname (&pass &secwhere);
			%if %length(&where) <= 0 && %length(%trim(&indexon)) > 0 %then %do;
				&indexon = _N_;
			%end;
			call symput("rc", "0");
		run;
		options obs=max nosyntaxcheck replace;
		&dbg;
		%if &rc = 1 %then %do;
			* subset was not successfully applied, try against master.;
			%if %length(%trim(&keyVar)) > 0 %then %do;
				* have a valid master column if it is needed.;
				data &outname.Temp(keep=&keyvar fmtname type label);
					if _N_ = 1 then call symput("rc", "2");
					set PUBDATA.&baseds (obs=max &pass &secwhere);
					retain fmtname 'secfmt' type 'c' label 'keep';
					call symput("rc", "0");
				run;
				options obs=max nosyntaxcheck replace;
				&dbg;
				%if &rc = 2 %then %do;
					proc format library=work;
						value $secfmt ''='notkeep';
					run;
					%let rc = 0;
				%end;
				%if &rc = 0 %then %do;
					proc freq data=&outname.Temp(rename=&keyVar=start);
						tables start * fmtname * type * label/noprint out=&outname.Temp(keep=start fmtname type label);
					run;
					proc format cntlin=&outname.Temp;
					run;
					data &outname.Temp;
						set &dsname(&pass where=(put(&keyVar,$secfmt.)='keep')); 
						%if %length(%trim(&indexon)) > 0 %then %do;
							&indexon = _N_;
						%end;
				%end;
			%end;
		%end;
		%else %do;
			* first subset attempt worked.;
			%if %length(%trim(&indexon)) > 0 %then %do;
				data &outname.Temp;
					set &outname.Temp;
					&indexon = _N_;
				run;
			%end;
		%end;
		%if &rc = 1 %then %do;
			* could not apply security subset.;
			data &outname.Temp;
				_HCMMSG = "&secmsgst";
				output;
			run;

			proc sql;
				create &outtype &outlib..&outname as select *
				from &outname.Temp;
			quit;
		%end;
		%else %do;
			* got a valid data set created.;
			proc sql;
				create &outtype &outlib..&outname as select 
				%if %length(%trim(&indexon)) > 0 %then %do;
					&indexon, 
				%end;
				%str(&bigcolst)
				from &outname.Temp (&obs) &where;
			quit;
		%end;
	%end;
	%else %do;
		* data set did not exist, put out fake data set/view with appropriate message in it.;
		data &outname.Temp;
			_HCMMSG = "&dnemsgst";
			output;
		run;

		proc sql;
			create &outtype  &outlib..&outname as select *
			from &outname.Temp; 
		quit;
	%end;
%mend;